'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _Dispatcher = require('../event/Dispatcher');

var _Dispatcher2 = _interopRequireDefault(_Dispatcher);

var _EventBus = require('../event/EventBus');

var _EventBus2 = _interopRequireDefault(_EventBus);

var _PageManager = require('../page/manager/PageManager');

var _PageManager2 = _interopRequireDefault(_PageManager);

var _PageStateManager = require('../page/state/PageStateManager');

var _PageStateManager2 = _interopRequireDefault(_PageStateManager);

var _Window = require('../window/Window');

var _Window2 = _interopRequireDefault(_Window);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Developer tools, used mostly for navigating the page state history.
 */
class DevTool {
  static get $dependencies() {
    return [_PageManager2.default, _PageStateManager2.default, _Window2.default, _Dispatcher2.default, _EventBus2.default];
  }

  /**
   * Initializes the developer tools.
   *
   * @param {PageManager} pageManager Application page manager.
   * @param {PageStateManager} stateManager Application state manager.
   * @param {Window} window IMA window wrapper.
   * @param {Dispatcher} dispatcher IMA event dispatcher.
   * @param {EventBus} eventBus IMA DOM event bus.
   */
  constructor(pageManager, stateManager, window, dispatcher, eventBus) {
    /**
     * Application page manager.
     *
     * @type {PageManager}
     */
    this._pageManager = pageManager;

    /**
     * Application state manager.
     *
     * @type {PageStateManager}
     */
    this._stateManager = stateManager;

    /**
     * IMA window wrapper.
     *
     * @type {Window}
     */
    this._window = window;

    /**
     * IMA event dispatcher.
     *
     * @type {Dispatcher}
     */
    this._dispatcher = dispatcher;

    /**
     * IMA DOM event bus.
     *
     * @type {EventBus}
     */
    this._eventBus = eventBus;
  }

  /**
   * Initializes the developer tools.
   */
  init() {
    if ($Debug) {
      if (this._window.isClient()) {
        this._window.getWindow().$IMA.$DevTool = this;
      }

      let window = this._window.getWindow();
      this._window.bindEventListener(window, 'keydown', e => {
        if (e.altKey && e.keyCode === 83) {
          // Alt + S
          console.log(this._stateManager.getState()); //eslint-disable-line no-console
        }
      });
    }
  }

  /**
   * Sets the provided state to the state manager.
   *
   * @param {Object<string, *>} statePatch A patch of the current page state.
   */
  setState(statePatch) {
    this._stateManager.setState(statePatch);
  }

  /**
   * Returns the current page state.
   *
   * @return {Object<string, *>} The current page state.
   */
  getState() {
    return this._stateManager.getState();
  }

  /**
   * Clears the current application state.
   */
  clearAppSource() {
    this._pageManager.destroy();
    this._dispatcher.clear();
  }
}
exports.default = DevTool; // @client-side

typeof $IMA !== 'undefined' && $IMA !== null && $IMA.Loader && $IMA.Loader.register('ima/debug/DevTool', [], function (_export, _context) {
 'use strict';
 return {
   setters: [],
   execute: function () {
     _export('default', exports.default);
   }
 };
});
