'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * An adapter component providing the current page controller's state to the
 * page view component through its properties.
 */
class ViewAdapter extends _react2.default.Component {
  /**
   * @inheritdoc
   * @return {{$Utils: function(*): ?Error}}
   */
  static get childContextTypes() {
    return {
      $Utils: _propTypes2.default.object.isRequired
    };
  }

  /**
   * Initializes the adapter component.
   *
   * @param {{
   *          state: Object<string, *>,
   *          view: function(new:React.Component, Object<string, *>)
   *        }} props Component properties, containing the actual page view
   *        and the initial page state to pass to the view.
   */
  constructor(props) {
    super(props.props);

    /**
     * The current page state as provided by the controller.
     *
     * @type {Object<string, *>}
     */
    this.state = props.state;

    /**
     * The actual page view to render.
     *
     * @type {function(new:React.Component, Object<string, *>)}
     */
    this._view = props.view;
  }

  /**
   * @inheritdoc
   */
  componentWillReceiveProps(newProps) {
    this.setState(newProps.state);
  }

  /**
   * @inheritdoc
   */
  render() {
    return _react2.default.createElement(this._view, this.state);
  }

  /**
   * @inheritdoc
   */
  getChildContext() {
    return {
      $Utils: this.props.$Utils
    };
  }
}
exports.default = ViewAdapter;

typeof $IMA !== 'undefined' && $IMA !== null && $IMA.Loader && $IMA.Loader.register('ima/page/renderer/ViewAdapter', [], function (_export, _context) {
 'use strict';
 return {
   setters: [],
   execute: function () {
     _export('default', exports.default);
   }
 };
});
