'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _PageStateManager = require('./PageStateManager');

var _PageStateManager2 = _interopRequireDefault(_PageStateManager);

var _GenericError = require('../../error/GenericError');

var _GenericError2 = _interopRequireDefault(_GenericError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Decorator for page state manager, which add logic for limiting Extension
 * competence.
 */
class PageStateManagerDecorator extends _PageStateManager2.default {
  /**
   * Initializes the page state manager decorator.
   *
   * @param {PageStateManager} pageStateManager
   * @param {string[]} allowedStateKeys
   */
  constructor(pageStateManager, allowedStateKeys) {
    super();

    /**
     * The current page state manager.
     *
     * @type {PageStateManager}
     */
    this._pageStateManager = pageStateManager;

    /**
     * Array of access keys for state.
     *
     * @type {string[]}
     */
    this._allowedStateKeys = allowedStateKeys;
  }

  /**
   * @inheritdoc
   */
  clear() {
    this._pageStateManager.clear();
  }

  /**
   * @inheritdoc
   */
  setState(statePatch) {
    if ($Debug) {
      let patchKeys = Object.keys(statePatch);
      let deniedKeys = patchKeys.filter(patchKey => {
        return this._allowedStateKeys.indexOf(patchKey) === -1;
      });

      if (deniedKeys.length > 0) {
        throw new _GenericError2.default(`Extension can not set state for keys ` + `${deniedKeys.join()}. Check your extension or add keys ` + `${deniedKeys.join()} to getAllowedStateKeys.`);
      }
    }

    this._pageStateManager.setState(statePatch);
  }

  /**
   * @inheritdoc
   */
  getState() {
    return this._pageStateManager.getState();
  }

  /**
   * @inheritdoc
   */
  getAllStates() {
    return this._pageStateManager.getAllStates();
  }
}
exports.default = PageStateManagerDecorator;

typeof $IMA !== 'undefined' && $IMA !== null && $IMA.Loader && $IMA.Loader.register('ima/page/state/PageStateManagerDecorator', [], function (_export, _context) {
 'use strict';
 return {
   setters: [],
   execute: function () {
     _export('default', exports.default);
   }
 };
});
